@@ -1,15 +1,16 @@
 // A go-plugin Server to write Rust-based plugins to Golang.
 
-mod error;
+pub mod error;
 mod grpc_broker;
+mod grpc_broker_service;
 mod grpc_controller;
 mod grpc_stdio;
-mod json_rpc_broker;
 mod unique_port;
+pub mod unix;
 
-pub use error::Error;
+use error::Error;
 
-use anyhow::anyhow;
+use anyhow::{anyhow, Context, Result};
 use http::{Request, Response};
 use hyper::Body;
 use std::clone::Clone;
@@ -18,26 +19,20 @@ use std::marker::Send;
 use tonic::body::BoxBody;
 use tonic::transport::NamedService;
 use tower::Service;
+use unix::TempSocket;
 
-pub use grpc_broker::grpc_plugins::ConnInfo;
-pub use json_rpc_broker::JsonRpcBroker;
+pub use grpc_broker::GRpcBroker;
+pub use grpc_broker_service::grpc_plugins::ConnInfo;
 pub use tonic::{Status, Streaming};
 
 use tokio::sync::mpsc::{unbounded_channel, UnboundedReceiver, UnboundedSender};
 
+pub type ServiceId = u32;
+
 // The constants are for generating the go-plugin string
 // https://github.com/hashicorp/go-plugin/blob/master/docs/guide-plugin-write-non-go.md
 const GRPC_CORE_PROTOCOL_VERSION: usize = 1;
 
-/// Golang/go-plugin don't support IPV6 yet. Yes, yes, I know...
-// bind to ALL addresses on Localhost
-const LOCALHOST_BIND_ADDR: &str = "0.0.0.0";
-
-// How should other processes on the localhost address localhost?
-const LOCALHOST_ADVERTISE_ADDR: &str = "127.0.0.1";
-
-const LOG_PREFIX: &str = "GrrPlugin::Server: ";
-
 pub struct HandshakeConfig {
     pub magic_cookie_key: String,
     pub magic_cookie_value: String,
@@ -52,6 +47,8 @@ pub struct Server {
     incoming_conninfo_stream_sender: UnboundedSender<Streaming<ConnInfo>>,
     incoming_conninfo_stream_receiver_receiver:
         UnboundedReceiver<UnboundedReceiver<Streaming<ConnInfo>>>,
+    trigger: triggered::Trigger,
+    listener: triggered::Listener,
 }
 
 impl Server {
@@ -67,14 +64,14 @@ impl Server {
         // using channels avoids having to do a complex sync dance using mutable globals.
         let (outgoing_conninfo_sender_transmitter, outgoing_conninfo_sender_receiver) =
             unbounded_channel();
-        log_and_escalate!(outgoing_conninfo_sender_transmitter.send(outgoing_conninfo_sender));
+        outgoing_conninfo_sender_transmitter.send(outgoing_conninfo_sender).context("Unable to send the outgoing_conninfo_sender to the transmitter. This is a tokio mpsc channel's transmitter being transmitted over another channel so it can be consumed exactly-one by someone later.")?;
 
         // Use this channel to send the channel receiver above from this constructor
         // to where it will be consumed in the "serve" method later...
         // using channels avoids having to do a complex sync dance using mutable globals.
         let (outgoing_conninfo_receiver_transmitter, outgoing_conninfo_receiver_receiver) =
             unbounded_channel();
-        log_and_escalate!(outgoing_conninfo_receiver_transmitter.send(outgoing_conninfo_receiver));
+        outgoing_conninfo_receiver_transmitter.send(outgoing_conninfo_receiver).context("Unable to send the outgoing_conninfo_receiver to the transmitter. This is a tokio mpsc channel's receiver being transmitted over another channel so it can be consumed exactly-one by someone later.")?;
 
         // Use this channel to send the channel from where we will receive ConnInfo's coming inbound
         // from the host, to the broker which will know what to do with them
@@ -89,9 +86,11 @@ impl Server {
             incoming_conninfo_stream_receiver_transmitter,
             incoming_conninfo_stream_receiver_receiver,
         ) = unbounded_channel();
-        log_and_escalate!(
-            incoming_conninfo_stream_receiver_transmitter.send(incoming_conninfo_stream_receiver)
-        );
+
+        incoming_conninfo_stream_receiver_transmitter.send(incoming_conninfo_stream_receiver)
+            .context("Unable to send the incoming_conninfo_stream_receiver to the transmitter. This is a tokio mpsc channel's receiver's receiver being transmitted over another channel so it can be consumed exactly-one by someone later. They will eventually listen to this channel to then get the actual stream over which they'll receive incoming ConnInfo's.")?;
+
+        let (trigger, listener) = triggered::trigger();
 
         Ok(Server {
             handshake_config,
@@ -100,13 +99,15 @@ impl Server {
             outgoing_conninfo_receiver_receiver,
             incoming_conninfo_stream_sender,
             incoming_conninfo_stream_receiver_receiver,
+            trigger,
+            listener,
         })
     }
 
-    pub async fn jsonrpc_broker(&mut self) -> Result<JsonRpcBroker, Error> {
+    pub async fn grpc_broker(&mut self) -> Result<GRpcBroker, Error> {
         let outgoing_conninfo_sender = match self.outgoing_conninfo_sender_receiver.recv().await {
             None => {
-                let err = anyhow!("{} jsonrpc_server_broker - jsonrpc_server_broker's transmission channel was None, which, being initalized in the constructor, was vended off already. Was this method called twice? Did someone else .recv() it?", LOG_PREFIX);
+                let err = anyhow!("jsonrpc_server_broker's transmission channel was None, which, being initalized in the constructor, was vended off already. Was this method called twice? Did someone else .recv() it?");
                 log::error!("{}", err);
                 return Err(Error::Other(err));
             }
@@ -119,7 +120,7 @@ impl Server {
             .await
         {
             None => {
-                let err = anyhow!("{} jsonrpc_server_broker - jsonrpc_server_broker's  receiver for a future incoming stream of ConnInfo was None, which, being initalized in the constructor, was vended off already.", LOG_PREFIX);
+                let err = anyhow!("jsonrpc_server_broker's receiver for a future incoming stream of ConnInfo was None, which, being initalized in the constructor, was vended off already.");
                 log::error!("{}", err);
                 return Err(Error::Other(err));
             }
@@ -127,38 +128,33 @@ impl Server {
         };
 
         // create the JSON-RPC 2.0 server broker
-        log::trace!(
-            "{}new -  Creating the JSON RPC 2.0 Server Broker.",
-            LOG_PREFIX
-        );
-        let jsonrpc_broker = JsonRpcBroker::new(
+        log::trace!("Creating the JSON RPC 2.0 Server Broker.",);
+        let jsonrpc_broker = GRpcBroker::new(
             unique_port::UniquePort::new(),
-            LOCALHOST_BIND_ADDR.to_string(),
-            LOCALHOST_ADVERTISE_ADDR.to_string(),
             outgoing_conninfo_sender,
             incoming_conninfo_stream_receiver,
+            self.listener.clone(),
         );
 
-        log::info!("{}new -  Created JSON RPC 2.0 Server Broker.", LOG_PREFIX);
+        log::info!("Created JSON RPC 2.0 Server Broker.");
 
         Ok(jsonrpc_broker)
     }
 
     // Copied from: https://github.com/hashicorp/go-plugin/blob/master/server.go#L247
     fn validate_magic_cookie(&self) -> Result<(), Error> {
-        log::info!("{}Validating the magic environment cookies to conduct the handshake. Expecting environment variable {}={}.",LOG_PREFIX, self.handshake_config.magic_cookie_key, self.handshake_config.magic_cookie_value);
+        log::info!("Validating the magic environment cookies to conduct the handshake. Expecting environment variable {}={}.", self.handshake_config.magic_cookie_key, self.handshake_config.magic_cookie_value);
         match env::var(&self.handshake_config.magic_cookie_key) {
             Ok(value) => {
                 if value == self.handshake_config.magic_cookie_value {
-                    log::info!("{}Handshake succeeded!", LOG_PREFIX);
+                    log::info!("Handshake succeeded!");
                     return Ok(());
                 } else {
-                    log::error!("{}Handshake failed due to environment variable {}'s value being {}, but expected to be {}.", LOG_PREFIX,self.handshake_config.magic_cookie_key, value, self.handshake_config.magic_cookie_value);
+                    log::error!("Handshake failed due to environment variable {}'s value being {}, but expected to be {}.",self.handshake_config.magic_cookie_key, value, self.handshake_config.magic_cookie_value);
                 }
             }
             Err(e) => log::error!(
-                "{}Handshake failed due to error reading environment variable {}: {:?}",
-                LOG_PREFIX,
+                "Handshake failed due to error reading environment variable {}: {:?}",
                 self.handshake_config.magic_cookie_key,
                 e
             ),
@@ -178,89 +174,66 @@ impl Server {
         <S as Service<http::Request<hyper::Body>>>::Error:
             Into<Box<dyn std::error::Error + Send + Sync>> + Send,
     {
-        log::trace!("{}serve - serving over a Tcp Socket...", LOG_PREFIX);
+        log::trace!("serving over a Tcp Socket...");
 
-        log_and_escalate!(self.validate_magic_cookie());
-
-        let (trigger, listener) = triggered::trigger();
+        self.validate_magic_cookie().context("Failed to validate magic cookie handshake from plugin client (i.e. host, i.e. consumer) to this Plugin.")?;
 
         let (mut health_reporter, health_service) = tonic_health::server::health_reporter();
         health_reporter.set_serving::<S>().await;
-        log::info!("{}serve -  gRPC Health Service created.", LOG_PREFIX);
+        log::info!("gRPC Health Service created.");
 
-        let service_port = match unique_port::UniquePort::new().get_unused_port() {
-            Some(p) => p,
-            None => {
-                let err =
-                    anyhow!("Unable to find a free unused TCP port to bind the gRPC server to");
-                log::error!("{}", err);
-                return Err(Error::Other(err));
-            }
-        };
-
-        log::info!("{}new - picked broker port: {}", LOG_PREFIX, service_port);
-
-        let addrstr = format!("{}:{}", LOCALHOST_BIND_ADDR, service_port);
-        let addr = log_and_escalate!(addrstr.parse());
+        let temp_socket = TempSocket::new()
+            .context("Failed to create a new TempSocket for opening the main gRPC listener to")?;
+        let socket_path = temp_socket.socket_filename()
+            .context("Failed to get a temporary socket filename from the temp socket for opening the main gRPC listener to")?;
+        log::trace!("Created new temp socket: {}", socket_path);
 
         let handshakestr = format!(
-            "{}|{}|tcp|{}:{}|grpc|",
-            GRPC_CORE_PROTOCOL_VERSION,
-            self.protocol_version,
-            LOCALHOST_ADVERTISE_ADDR,
-            service_port
+            "{}|{}|unix|{}|grpc|",
+            GRPC_CORE_PROTOCOL_VERSION, self.protocol_version, socket_path,
         );
+        log::trace!("Created Handshake string: {}", handshakestr);
 
-        log::trace!(
-            "{}serve - Created Handshake string: {}",
-            LOG_PREFIX,
-            handshakestr
-        );
+        // create incoming stream from unix socket above...
+        let incoming_stream_from_socket = unix::incoming_from_path(socket_path.as_str()).await?;
+        log::trace!("Created Incoming unix stream from the socket");
 
         let outgoing_conninfo_receiver = match self.outgoing_conninfo_receiver_receiver.recv().await {
             Some(outgoing_conninfo_receiver) => outgoing_conninfo_receiver,
             None => return Err(Error::Other(anyhow!("Outgoing ConnInfo receiver does not exist. Did someone else .recv() it before? It was created in the constructor, so should be available in the method."))),
         };
 
-        log::info!("{} serve - Creating a GRPC Broker Server.", LOG_PREFIX);
+        log::info!("Creating a GRPC Broker Server.");
         // mspc Senders can be cloned. Receivers need all the attention and queueing.
-        let broker_server = grpc_broker::new_server(
+        let broker_server = grpc_broker_service::new_server(
             outgoing_conninfo_receiver,
             self.incoming_conninfo_stream_sender.clone(),
         )
         .await?;
 
-        log::info!("{} serve - Creating a GRPC Controller Server.", LOG_PREFIX);
-        let controller_server = grpc_controller::new_server(trigger);
-        log::info!("{} serve - Creating a GRPC Stdio Server.", LOG_PREFIX);
+        log::info!("Creating a GRPC Controller Server.");
+        let controller_server = grpc_controller::new_server(self.trigger.clone());
+        log::info!("Creating a GRPC Stdio Server.");
         let stdio_server = grpc_stdio::new_server();
 
-        log::info!("{}serve - Starting service...", LOG_PREFIX);
-
+        let listener = self.listener.clone();
+        log::info!("Starting service...");
         let grpc_service_future = tonic::transport::Server::builder()
             .add_service(health_service)
             .add_service(broker_server)
             .add_service(controller_server)
             .add_service(stdio_server)
             .add_service(plugin)
-            .serve_with_shutdown(addr, async { listener.await });
+            .serve_with_incoming_shutdown(incoming_stream_from_socket, async { listener.await });
 
-        log::info!(
-            "{}About to print handshake string: {}",
-            LOG_PREFIX,
-            handshakestr
-        );
-        println!("{}                        \n\n", handshakestr);
+        log::info!("About to print handshake string: {}", handshakestr);
+        println!("{}", handshakestr);
 
         // starting broker and plugin services now...
         //join!(broker_service_future, plugin_service_future);
         let result = grpc_service_future.await;
 
-        log::info!(
-            "{}gRPC broker service ended with result: {:?}",
-            LOG_PREFIX,
-            result
-        );
+        log::info!("gRPC broker service ended with result: {:?}", result);
 
         Ok(())
     }
