@@ -1,7 +1,5 @@
 use portpicker::Port;
 
-const LOG_PREFIX: &str = "GrrPlugin::UniquePort: ";
-
 pub struct UniquePort {
     vended_ports: Vec<Port>,
 }
@@ -27,10 +25,10 @@ impl UniquePort {
                 None => return None,
                 Some(p) => {
                     if self.vended_ports.contains(&p) {
-                        log::trace!("{} - Skipped port: {} because it is in the list of previously vended ports: {:?}", LOG_PREFIX, p, self.vended_ports);
+                        log::trace!("Skipped port: {} because it is in the list of previously vended ports: {:?}", p, self.vended_ports);
                         continue;
                     } else {
-                        log::trace!("{} - Vending port: {}", LOG_PREFIX, p);
+                        log::trace!("Vending port: {}", p);
                         self.vended_ports.push(p);
                         return Some(p);
                     }
